/*
***************************************************************************
*   Copyright (C) 2008 Structural Proteomics Group, Max Planck Institute  *
*   for Molecular Genetics, Berlin, Germany                               *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************
*/
package actionTools;

/**
 * @author Lars Petzold
 * 
 * The purpose of this class is to provide an abstract strategy to retrieve
 * data from a source and store it in target object. Therefore, you have to 
 * implement function <code>retrieve(Object obj)</code> as it suits best your 
 * requirements. The data to be stored might violate the storable data type,
 * hence, this function throws <code>java.lang.ClassCastExceptions</code>.
 */

public abstract class Retriever extends Action {
    public Retriever(Object obj) {
	super(obj);
    }
    public abstract void retrieve(Object obj) throws ClassCastException;
}
