/*
***************************************************************************
*   Copyright (C) 2008 Structural Proteomics Group, Max Planck Institute  *
*   for Molecular Genetics, Berlin, Germany                               *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************
*/
package actionTools;

/**
 * @author Lars Petzold
 * 
 * Abstract class to implement "inline runnables" for the thread executer by 
 * implementing function {@link #implRun()}.
 */
public abstract class Runner implements Runnable {
	
	/**
	 * Function to be implemented. This function is invoked in function 
	 * {@link #run()} and, therefore, is designated to provide everything to be 
	 * done in the thread.
	 */
	public abstract void implRun();
	
	/**
	 * The actual runner. Invokes function {@link #implRun()}.
	 */
	public void run() {
		implRun();
	}

}
