/*
***************************************************************************
*   Copyright (C) 2008 Structural Proteomics Group, Max Planck Institute  *
*   for Molecular Genetics, Berlin, Germany                               *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************
*/
package cmview;
/**
 * Helper class for LoadDialog.
 * This abstract class can be instantianted and passed to a LoadDialog to specify the action that
 * will happen if the ok button is pressed. It contains a constructor
 * and an abstract doit() function which is called by LoadDialog.
 */

public abstract class LoadAction {
	
	boolean secondModel;	// flag which we use to distinguish between first and second model being loaded
	
	public LoadAction(boolean secondModel) {
		this.secondModel = secondModel;
	}
	
	/**
	 * Loads the chain from the given source. Some of these arguments might be 
	 * null according to the chosen source (e.g. if the source is a local pdb 
	 * file the database name does not need to be present).
	 * @param o  parent
	 * @param f  filename
	 * @param ac  accession code
	 * @param modelSerial the model serial
	 * @param cc  chain code
	 * @param ct  contact type
	 * @param dist  distance threshold for the contacts
	 * @param minss  minimal sequence separation
	 * @param maxss  maximal sequence separation
	 * @param db  name of the database
	 * @param gid  graph id
	 */
	public abstract void doit(Object o, String f, String ac, int modelSerial, String cc, String ct, double dist, int minss, int maxss, String db, int gid);

}
