/*
***************************************************************************
*   Copyright (C) 2008 Structural Proteomics Group, Max Planck Institute  *
*   for Molecular Genetics, Berlin, Germany                               *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************
*/
package cmview;
import java.awt.*;
import javax.swing.*;
import java.awt.print.*;

/**
 * Helper class for printing Swing components.
 * Encapsulates the code for printing a Swing component using a print dialog.
 * Usage: Call static printComponent(JComponent) method.
 * Taken from some online Java tutorial.
 */
public class PrintUtil implements Printable {
  private Component componentToBePrinted;

  public static void printComponent(Component c) {
    new PrintUtil(c).print();
  }
  
  private PrintUtil(Component componentToBePrinted) {
    this.componentToBePrinted = componentToBePrinted;
  }
  
  private void print() {
    PrinterJob printJob = PrinterJob.getPrinterJob();
    printJob.setPrintable(this);
    if (printJob.printDialog())
      try {
        printJob.print();
      } catch(PrinterException pe) {
        System.out.println("Error printing: " + pe);
      }
  }

  /** Overloaded method to implement printable interface */
  public int print(Graphics g, PageFormat pageFormat, int pageIndex) {
    if (pageIndex > 0) {
      return(NO_SUCH_PAGE);
    } else {
      Graphics2D g2d = (Graphics2D)g;
      g2d.translate(pageFormat.getImageableX(), pageFormat.getImageableY());
      // Set the right size for printing in ContactMapPane
      disableDoubleBuffering(componentToBePrinted);
      if(componentToBePrinted instanceof ContactMapPane) {
    	ContactMapPane p = (ContactMapPane) componentToBePrinted;
        double h = pageFormat.getImageableHeight();
        double w = pageFormat.getImageableWidth();
        p.print(w,h,g2d);
      } else {
    	  componentToBePrinted.paint(g2d);
      }
      enableDoubleBuffering(componentToBePrinted);
      // TODO: Reset the right size for screen output in ContactMapPane
      return(PAGE_EXISTS);
    }
  }

  private static void disableDoubleBuffering(Component c) {
    RepaintManager currentManager = RepaintManager.currentManager(c);
    currentManager.setDoubleBufferingEnabled(false);
  }

  private static void enableDoubleBuffering(Component c) {
    RepaintManager currentManager = RepaintManager.currentManager(c);
    currentManager.setDoubleBufferingEnabled(true);
  }
}