/*
***************************************************************************
*   Copyright (C) 2008 Structural Proteomics Group, Max Planck Institute  *
*   for Molecular Genetics, Berlin, Germany                               *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************
*/
package cmview.sadpAdapter;

import javax.swing.JButton;

public class SADPDialogDoneNotifier extends JButton {
    static final long serialVersionUID = 1l;
    Integer    status = SADPDialog.IDLE;
    SADPRunner runner;
    SADPResult result;
    
    public SADPDialogDoneNotifier(SADPRunner runner) {
	this.runner = runner;
    }
    
    public SADPDialogDoneNotifier(SADPResult result) {
    	this.result = result;
    }
    
    /**
     * Set the notification status for this notifier.
     * @param status
     * @see #secureNotify(Integer)
     */
    public void notify(Integer status) {
	this.status = status;
	doClick();
    }
    
    /**
     * Sets the notification status for this notifier in a secure way: If the 
     * current status differs from <code>ToolDialog.IDLE</code> the given 
     * status will not be stored.
     * @param status  the new status
     */
    public void secureNotify(Integer status) {
	if(this.status == SADPDialog.IDLE) {
	    this.status = status;
	    doClick();
	}
    }

    public Integer notification() {
	return status;
    }
        
    public SADPRunner getRunner() {
	return runner;
    }
    
    public SADPResult getResult() {
    	return result;
    }
}
