/*
***************************************************************************
*   Copyright (C) 2008 Structural Proteomics Group, Max Planck Institute  *
*   for Molecular Genetics, Berlin, Germany                               *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************
*/
package cmview.toolUtils;

import java.util.concurrent.Callable;

import actionTools.Action;

/**
 * Basic class for any tool that should run in a separate thread.
 *  
 * TODO: Maybe it is more reasonable to implement the Callable interface instead of the Runnable interface as 
 * this support direct results retrieval.
 * 
 * @author Lars Petzold
 *
 */
public abstract class ToolRunner<V> implements Callable<V>{
    
    boolean done = false;
    Action  actionWhenDone;
    
    /**
     * Empty constructor.
     */
    public ToolRunner() {
	
    }
    
    /**
     * Creates a new tool runner along with the an action-when-done instance.
     * @param toBePerformedWhenDone
     */
    public ToolRunner(Action toBePerformedWhenDone) {
	actionWhenDone = toBePerformedWhenDone;
    }
    
    /**
     * Gets the action object.
     * @return
     */
    public Action getActionWhenDone() {
	return actionWhenDone;
    }
    
    /**
     * Action to be performed when the runner finishes his work.
     * @param toBePerformedWhenDone
     */
    public void setActionWhenDone(Action toBePerformedWhenDone) {
	actionWhenDone = toBePerformedWhenDone;
    }
    
    /**
     * Checks whether tool runner has finished his work.
     * @return true if so, else false
     * @deprecated
     */
    public boolean isDone() {
	return done;
    }
    
    /**
     * Set state of tool runner.
     * @param b  true if he has finished his work, else false.
     * @deprecated
     */
    public void isDone( boolean b ) {
	done = b;
    }
}
