/*
***************************************************************************
*   Copyright (C) 2008 Structural Proteomics Group, Max Planck Institute  *
*   for Molecular Genetics, Berlin, Germany                               *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************
*/
package proteinstructure;

public class AIGNode {
	
	private int atomSerial;
	private String atomName;
	private RIGNode parentResidue;

	public AIGNode(int atomSerial, String atomName, RIGNode parentResidue) {
		this.atomSerial = atomSerial;
		this.atomName = atomName;
		this.parentResidue = parentResidue;
	}
	
	public int getAtomSerial() {
		return this.atomSerial;
	}
	
	public String getAtomName() {
		return this.atomName;
	}
	
	public RIGNode getParent() {
		return this.parentResidue;
	}
	
	/**
	 * Convenience method to get the residue serial for this atom node
	 * @return
	 */
	public int getResidueSerial() {
		return this.parentResidue.getResidueSerial();
	}
	
	public boolean equals(Object obj) {
		if (obj instanceof AIGNode) {
			AIGNode v = (AIGNode) obj;
			return ((this.parentResidue.equals(v.getParent())) && (this.atomSerial == v.getAtomSerial()) && (this.atomName.equals(v.getAtomName())));
		} else {
			return false;
		}
	}
	
	public AIGNode copy() {
		return (new AIGNode(atomSerial, atomName, this.parentResidue.copy()));
	}
}
