/*
***************************************************************************
*   Copyright (C) 2008 Structural Proteomics Group, Max Planck Institute  *
*   for Molecular Genetics, Berlin, Germany                               *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************
*/
package proteinstructure;

import java.util.HashMap;
import java.util.Set;

/**
 * A particular catalytic within a protein structure
 */
public class CatalyticSite {
	
	/*--------------------------- member variables --------------------------*/

	HashMap<Integer,String> resser2chemfunc;
	int id;
	String evidence;
	String littEntryPdbCode;
	String littEntryPdbChainCode;
	
	/*----------------------------- constructors ----------------------------*/
	
	public CatalyticSite(int id, String evidence, String littEntryPdbCode, String littEntryPdbChainCode) {
		this.id = id;
		this.evidence = evidence;
		this.littEntryPdbCode = littEntryPdbCode;
		this.littEntryPdbChainCode = littEntryPdbChainCode;
		resser2chemfunc = new HashMap<Integer,String>();
	}
	
	/*---------------------------- public methods ---------------------------*/
	
	public CatalyticSite copy() {
		return new CatalyticSite(id, evidence, littEntryPdbCode, littEntryPdbChainCode);
	}
	
	// add a residue with specific chemical function as a catalytic one
	public void addRes(int resser, String chemFunc) {
		resser2chemfunc.put(resser, chemFunc);
	}
	
	// remove a residue
	public void remRes(int resser) {
		resser2chemfunc.remove(resser);
	}
	
	// get all residues of the catalytic site
	public Set<Integer> getRes() {
		return resser2chemfunc.keySet();
	}

	// get the chemical function from residue serial
	public String getChemFuncFromResSerial(int resser) {
		return resser2chemfunc.get(resser);
	}
	
	/** Returns the serial for this site */ 
	public int getSerial() {
		return id;
	}
	
	/** Returns the evidence for this site */ 
	public String getEvidence() {
		return evidence;
	}
	
	/** Returns the literature entry pdb code for this site */ 
	public String getLittEntryPdbCode() {
		return littEntryPdbCode;
	}
	
	/** Returns the literature entry pdb chain code for this site */ 
	public String getLittEntryPdbChainCode() {
		return littEntryPdbChainCode;
	}
	
	public void print() {
		for(int resSer:resser2chemfunc.keySet()) {
			System.out.println("site:"+id+"-res:"+resSer+"-func:"+getChemFuncFromResSerial(resSer));
		}
	}
}

