/*
***************************************************************************
*   Copyright (C) 2008 Structural Proteomics Group, Max Planck Institute  *
*   for Molecular Genetics, Berlin, Germany                               *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************
*/
package proteinstructure;

import java.util.Set;
import java.util.TreeMap;


/**
 * Class representing a contact type, i.e. the atoms that belong to each residue for a given contact type
 * Data hold in a TreeMap structure with keys the three letter residue types and values sets of atom names
 *
 */
public class ContactType extends TreeMap<String,Set<String>> {

	private static final long serialVersionUID = 1L;
	
	private String name;
	private boolean multiAtom; // true when multiAtom type, false when singleAtom type
	
	public ContactType(String name, boolean multiAtom) {
		super();
		this.name = name;
		this.multiAtom = multiAtom;
	}
	
	public boolean isMultiAtom() {
		return multiAtom;
	}
	
	public String getName() {
		return name;
	}
	
}
