/*
***************************************************************************
*   Copyright (C) 2008 Structural Proteomics Group, Max Planck Institute  *
*   for Molecular Genetics, Berlin, Germany                               *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************
*/
package proteinstructure;

/**
 * A particular EC region within a protein structure
 */
public class ECRegion {
	
	/*--------------------------- member variables --------------------------*/
	
	String id;						// the EC number
	Interval interval;				// the location of this region in the sequence
	String startPdbRes, endPdbRes; 	// the starting and ending pdb residue serials of this region in the sequence
	
	/*----------------------------- constructors ----------------------------*/
	
	public ECRegion(String id, String startPdbRes, String endPdbRes, int startRes, int endRes) {
		this.id = id;
		this.interval = new Interval(startRes, endRes);
		this.startPdbRes = startPdbRes;
		this.endPdbRes = endPdbRes;
	}
	
	/*---------------------------- public methods ---------------------------*/
	
	public ECRegion copy() {
		return new ECRegion(id, startPdbRes, endPdbRes, interval.beg, interval.end);
	}
	
	/** Returns the ec number of this region */
	public String getECNum() {
		return id;
	}
	
	/** Returns the starting pdb residue serial of this region */ 
	public String getStartPdbRes() {
		return startPdbRes;
	}

	/** Returns the ending pdb residue serial of this region */ 
	public String getEndPdbRes() {
		return endPdbRes;
	}

	/** Returns the range of this region in the sequence. */ 
	public Interval getInterval() {
		return interval;
	}
	
	/*---------------------------- static methods ---------------------------*/
}	
