/*
***************************************************************************
*   Copyright (C) 2008 Structural Proteomics Group, Max Planck Institute  *
*   for Molecular Genetics, Berlin, Germany                               *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************
*/
package proteinstructure;

import java.util.Comparator;

import edu.uci.ics.jung.graph.util.Pair;

/**
 * A Comparator for Pair<Integer>, it can be used in constructing sorted 
 * collections of Pair<Integer>. Pair doesn't implement comparable, that's why
 * this is needed 
 * 
 * Example use: 
 * 
 * TreeMap<Pair<Integer>> map = new TreeMap<Pair<Integer>>(new IntPairComparator());
 *
 */
public class IntPairComparator implements Comparator<Pair<Integer>> {

	public int compare(Pair<Integer> pair1, Pair<Integer> pair2) {
		if (pair1.getFirst()>pair2.getFirst()){
			return 1;
		} 
		else if (pair1.getFirst()<pair2.getFirst()){
			return -1;
		}
		else {
			if (pair1.getSecond()>pair2.getSecond()){
				return 1;
			}
			else if (pair1.getSecond()<pair2.getSecond()){
				return -1;
			}
		}				
		return 0;
	}

}
