/*
***************************************************************************
*   Copyright (C) 2008 Structural Proteomics Group, Max Planck Institute  *
*   for Molecular Genetics, Berlin, Germany                               *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************
*/
package proteinstructure;

import java.util.TreeMap;
import java.util.Collections;

public class RIGCommonNbhood extends TreeMap<Integer,RIGNode> {
	

	private static final long serialVersionUID = 1L;

	private RIGNode iNode;
	private RIGNode jNode;
	private boolean connected;
	
	public RIGCommonNbhood(RIGNode iNode, RIGNode jNode, boolean connected){
		super();
		this.iNode = iNode;
		this.jNode = jNode;
		this.connected = connected;
	}

	public RIGNode getFirstNode() {
		return iNode;
	}
	
	public RIGNode getSecondNode() {
		return jNode;
	}
	
	public boolean areNodesConnected() {
		return connected;
	}
	
	public String getMotif(){
		String motif="";
		int min=Math.min(Math.min(iNode.getResidueSerial(),jNode.getResidueSerial()), Collections.min(this.keySet()));
		int max=Math.max(Math.max(iNode.getResidueSerial(),jNode.getResidueSerial()), Collections.max(this.keySet()));
		int gapSize=0;
		String gap="";
		for (int i=min;i<=max;i++){
			if (this.containsKey(i)){
				motif+=gap;
				motif+=AAinfo.threeletter2oneletter(this.get(i).getResidueType());
				gapSize=0;
				gap="";
			} else if (i==iNode.getResidueSerial()){
				motif+=gap;
				motif+="x";
				gapSize=0;
				gap="";
			} else if (i==jNode.getResidueSerial()){
				motif+=gap;
				motif+="y";
				gapSize=0;
				gap="";
			} else {
				gapSize++;
				gap="_{"+gapSize+"}";
			}
		}
		return motif;
	}

	public String getMotifFullGaps(){
		String motif="";
		int min=Math.min(Math.min(iNode.getResidueSerial(),jNode.getResidueSerial()), Collections.min(this.keySet()));
		int max=Math.max(Math.max(iNode.getResidueSerial(),jNode.getResidueSerial()), Collections.max(this.keySet()));
		for (int i=min;i<=max;i++){
			if (this.containsKey(i)){
				motif+=AAinfo.threeletter2oneletter(this.get(i).getResidueType());	
			} else if (i==iNode.getResidueSerial()){
				motif+="x";
			} else if (i==jNode.getResidueSerial()){
				motif+="y";				
			} else {
				motif+="_";
			}
		}
		return motif;
	}
	
	public String getCommaSeparatedResSerials(){
		String ressers="";
		for (int resser:this.keySet()){
			ressers += resser+","; 
		}
		// we chop off the last comma
		if (ressers.length()!=0) { 
			ressers = ressers.substring(0, ressers.length()-1);
		}
		return ressers;
	}
	
	public String toString(){
		if (this.isEmpty()) return "";
		else return this.getMotif();
	}
}
