/*
***************************************************************************
*   Copyright (C) 2008 Structural Proteomics Group, Max Planck Institute  *
*   for Molecular Genetics, Berlin, Germany                               *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************
*/
package proteinstructure;

/**
 * Class representing a Residue Interaction Graph edge.
 * The RIGEdge stores properties of the edge (like weights) but not the 
 * end points. Those can be taken from the RIGraph with getEndPoints(edge)
 */
public class RIGEdge {

	private static double DEFAULT_WEIGHT = 1.0;
	private static int DEFAULT_ATOM_WEIGHT = 1;
	
	private double weight;
	private int atomWeight;  
	private double distance;
	
	public RIGEdge(double weight, double distance, int atomWeight) {
		this.weight = weight;
		this.distance = distance;
		this.atomWeight = atomWeight;
	}
	
	public RIGEdge(int atomWeight){
		this.atomWeight = atomWeight;
		this.distance = 0;
		this.weight = DEFAULT_WEIGHT;
	}
	
	public RIGEdge(double weight) {
		this.weight=weight;
		this.distance = 0;
		this.atomWeight = DEFAULT_ATOM_WEIGHT;
	}
	
	public RIGEdge() {
		this.weight=DEFAULT_WEIGHT;
		this.distance = 0;
		this.atomWeight = DEFAULT_ATOM_WEIGHT;
	}
	
	public int getAtomWeight() {
		return atomWeight;
	}
	
	public void setAtomWeight(int atomWeight) {
		this.atomWeight = atomWeight;
	}

	public double getWeight() {
		return weight;
	}
	
	public void setWeight(double weight) {
		this.weight = weight;
	}
	
	public double getDistance() {
		return distance;
	}
	
	public void setDistance(double distance) {
		this.distance = distance;
	}

	public String toString() {
		return "E";
	}
	
	public RIGEdge copy() {
		return new RIGEdge(this.weight,this.distance,this.atomWeight);
	}
	
}
