/*
***************************************************************************
*   Copyright (C) 2008 Structural Proteomics Group, Max Planck Institute  *
*   for Molecular Genetics, Berlin, Germany                               *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************
*/
package proteinstructure;

/**
 * Class representing a Residue Interaction Graph node, i.e. a residue
 * 
 */
public class RIGNode {

	private int residueSerial;
	private String residueType;
	private SecStrucElement sselem;
	
	
	public RIGNode(int residueSerial, String residueType, SecStrucElement sselem) {
		this.residueSerial = residueSerial;
		this.residueType = residueType;
		this.sselem = sselem;
	}
	
	public RIGNode(int residueSerial, String residueType) {
		this.residueSerial = residueSerial;
		this.residueType = residueType;
		this.sselem = null;
	}
	
	public RIGNode(int residueSerial) {
		this.residueSerial = residueSerial;
		this.residueType = null;
		this.sselem = null;
	}

	public int getResidueSerial() {
		return residueSerial;
	}

	public String getResidueType() {
		return residueType;
	}
	
	public SecStrucElement getSecStrucElement() {
		return sselem;
	}
	
	public String toString() {
		return residueSerial+AAinfo.threeletter2oneletter(residueType);
	}
	
	/**
	 * Deep copies this RIGNode
	 * @return
	 */
	public RIGNode copy() {
		SecStrucElement newsselem = this.sselem==null?null:this.sselem.copy();
		RIGNode copy = new RIGNode(this.residueSerial, this.residueType, newsselem);
		return copy;
	}
	
	public boolean equals(Object obj) {
		if (obj instanceof RIGNode) {
			RIGNode v = (RIGNode) obj;
			return ((this.residueSerial == v.getResidueSerial()) && (this.residueType.equals(v.getResidueType())));
		} else {
			return false;
		}
	}
	
}
